@@ -901,24 +901,14 @@ void catdelete()
       }
       else { 		/* All's clear to delete.		     	   */
 
-         /* First remove any instances of this object in the delete buffer */
+         /* Clear the undo stack so that any references to this object	*/
+	 /* won't screw up the database (this could be kinder & gentler	*/
+	 /* by first checking if there are any references to the object	*/
+	 /* in the undo stack. . .					*/
 
-         for (compobj = xobjs.delbuffer.library; compobj != xobjs.delbuffer.library +
-                xobjs.delbuffer.number; compobj++) {
-	    for (testobj = (*compobj)->plist; testobj < (*compobj)->plist
-	          + (*compobj)->parts; testobj++) {
-	       if ((*testobj)->type == OBJECT) {
-	          if (TOOBJINST(testobj)->thisobject == libobj->thisobject) {
-		     free(TOOBJINST(testobj));
-		     for (tobj = testobj; tobj < (*compobj)->plist
-			   + (*compobj)->parts - 1; tobj++) {
-		        (*tobj) = (*(tobj + 1));
-		     }
-		     (*compobj)->parts--;
-		  }
-	       }
-	    }
-         }
+	 flush_undo_stack();
+
+	 /* Next, remove the object from the library page. */
 
 	 for (tlib = libpage; tlib < libpage + *libpobjs; tlib++)
 	    if ((*tlib) == libobj->thisobject) {
