@@ -397,8 +397,12 @@ void autostop(xcWidget w, caddr_t clientdata, caddr_t nulldata)
 #endif
 
 /*--------------------------------------------------------------*/
-/* Auto-scale							*/
+/* Auto-scale the drawing to fit the declared page size.	*/
 /* (for now, with fixed 1" margins)				*/
+/*								*/
+/* If the page is declared encapsulated, then do nothing.	*/
+/* If a frame box is on the page, then scale to fit the frame	*/
+/* to the declared page size, not the whole object.		*/
 /*--------------------------------------------------------------*/
 
 void autoscale(int page)
@@ -408,6 +412,7 @@ void autoscale(int page)
 	== CM) ? CMSCALE : INCHSCALE;
    float margin = 144.0;
    int width, height;
+   polyptr framebox;
 
    if (!(xobjs.pagelist[page]->pmode & 2)) return;
    else if (topobject->bbox.width == 0 || topobject->bbox.height == 0) {
@@ -420,13 +425,25 @@ void autoscale(int page)
    newyscale = (xobjs.pagelist[page]->pagesize.y - margin)
 	/ scalefudge;
 
-#ifdef SCHEMA
-   width = toplevelwidth(areastruct.topinstance);
-   height = toplevelheight(areastruct.topinstance);
-#else
-   width = topobject->bbox.width;
-   height = topobject->bbox.height;
-#endif
+   if ((framebox = checkforbbox(topobject)) != NULL) {
+      int i, minx, miny, maxx, maxy;
+      
+      minx = maxx = framebox->points[0].x;
+      miny = maxy = framebox->points[0].y;
+      for (i = 1; i < framebox->number; i++) {
+	 if (framebox->points[i].x < minx) minx = framebox->points[i].x;
+	 else if (framebox->points[i].x > maxx) maxx = framebox->points[i].x;
+	 if (framebox->points[i].y < miny) miny = framebox->points[i].y;
+	 else if (framebox->points[i].y > maxy) maxy = framebox->points[i].y;
+      }
+      width = (maxx - minx);
+      height = (maxy - miny);
+   }
+   else {
+
+      width = toplevelwidth(areastruct.topinstance, NULL);
+      height = toplevelheight(areastruct.topinstance, NULL);
+   }
 
    if (xobjs.pagelist[page]->orient == 0) {	/* Portrait */
       newxscale /= width;
@@ -436,7 +453,6 @@ void autoscale(int page)
       newxscale /= height;
       newyscale /= width;
    }
-
    xobjs.pagelist[page]->outscale = min(newxscale, newyscale);
 }
 
@@ -1027,11 +1043,9 @@ void setpagelabel(xcWidget w, char *dataptr)
    else
       sprintf(topobject->name, "%.79s", _STR2);
 
-#ifdef SCHEMA
    /* For schematics, all pages with associations to symbols must have	*/
    /* unique names.							*/
    if (topobject->symschem != NULL) checkpagename(topobject);
-#endif
 
    printname(topobject);
    renamepage(areastruct.page);
@@ -1109,7 +1123,8 @@ void makenewencodingbutton(char *ename, char value)
 
 void togglefontmark(int fontval)
 {
-   Tcl_SetVar(xcinterp, "fontfamily", fonts[fontval].family, TCL_NAMESPACE_ONLY);
+   Tcl_SetVar2(xcinterp, "XCOps", "fontfamily", fonts[fontval].family,
+		TCL_NAMESPACE_ONLY);
 }
 
 #else
@@ -1193,13 +1208,7 @@ void toggleexcl(xcWidget widget, menuptr menu, int menulength)
 /* Toggle a menu item */
 /*--------------------*/
 
-#ifdef TCL_WRAPPER
-
-void toggle(xcWidget w, Boolean *boolvalue, caddr_t calldata)
-{
-}
-
-#else
+#ifndef TCL_WRAPPER
 
 void toggle(xcWidget w, Boolean *boolvalue, caddr_t calldata)
 {
@@ -1291,7 +1300,7 @@ void togglegrid(u_short type)
    /* string; doing so prevents the menu from recognizing the variable	*/
    /* value.								*/
 
-   Tcl_SetVar(xcinterp, "gridstyle", stylenames[type], TCL_NAMESPACE_ONLY);
+   Tcl_SetVar2(xcinterp, "XCOps", "gridstyle", stylenames[type], TCL_NAMESPACE_ONLY);
 }
 
 #else
@@ -1537,12 +1546,15 @@ void makepagebutton()
 
 void renamepage(short pagenumber)
 {
+   int page;
+   objinstptr thisinst = xobjs.pagelist[pagenumber]->pageinst;
+
 #ifdef TCL_WRAPPER
    char *pname, *plabel;
 
    if ((pagenumber >= 0) && (pagenumber < xobjs.pages - 1) &&
-	    (xobjs.pagelist[pagenumber]->pageinst != NULL)) {
-      plabel = xobjs.pagelist[pagenumber]->pageinst->thisobject->name;
+	    (thisinst != NULL)) {
+      plabel = thisinst->thisobject->name;
       pname = (char *)malloc(28 + strlen(plabel));
       sprintf(pname, "xcircuit::renamepage %d {%s}", pagenumber + 1, plabel);
       Tcl_Eval(xcinterp, pname);
@@ -1557,10 +1569,10 @@ void renamepage(short pagenumber)
    sprintf(bname, "Page %d", pagenumber + 1);
    button = XtNameToWidget(parent, bname);
 
-   if ((button != NULL) && (xobjs.pagelist[pagenumber]->pageinst != NULL)) {
-      if (xobjs.pagelist[pagenumber]->pageinst->thisobject->name != NULL)
+   if ((button != NULL) && (thisinst != NULL)) {
+      if (thisinst->thisobject->name != NULL)
          XtSetArg(wargs[0], XtNlabel,
-		xobjs.pagelist[pagenumber]->pageinst->thisobject->name);
+		thisinst->thisobject->name);
       else
          XtSetArg(wargs[0], XtNlabel, bname);
       XtSetValues(button, wargs, 1);
@@ -1568,6 +1580,34 @@ void renamepage(short pagenumber)
    else if (button == NULL)
       Fprintf(stderr, "Error:  No Button Widget named \"%9s\"\n", bname);
 #endif
+
+   /* If the page had a master but now has a different	*/
+   /* name, disconnect the schematics.			*/
+
+   if (thisinst->thisobject->schemtype == SECONDARY) {
+      if (strcmp(thisinst->thisobject->name, thisinst->thisobject->symschem->name)) {
+	 thisinst->thisobject->symschem = NULL;
+	 thisinst->thisobject->schemtype = PRIMARY;
+      }
+      Wprintf("Page disconnected from master schematic.");
+   }
+
+   /* Conversely, if the page has the same name as another page, the	*/
+   /* schematics are considered connected.				*/
+
+   for (page = 0; page < xobjs.pages; page++) {
+      objinstptr pageinst;
+      if (page == pagenumber) continue;
+      pageinst = xobjs.pagelist[page]->pageinst;
+      if (pageinst && !strcmp(pageinst->thisobject->name,
+		thisinst->thisobject->name)) {
+	 thisinst->thisobject->symschem = pageinst->thisobject;
+	 thisinst->thisobject->schemtype = SECONDARY;
+         Wprintf("Page connected to master schematic.");
+	 break;
+      }
+   }
+
 }
 
 /*--------------------------------------------------------------*/
@@ -1647,21 +1687,22 @@ void setcolormark(int colorval)
    /* Set GUI variables and execute any command tags associated */
    /* with the "color" command */
 
-   Tcl_Obj *objv[2];
+   Tcl_Obj *objv[3];
    int i;
 
    objv[0] = Tcl_NewStringObj("color", 5);
+   objv[1] = Tcl_NewStringObj("set", 3);
    if (colorval == DEFAULTCOLOR)
-      objv[1] = Tcl_NewStringObj("inherit", 7);
+      objv[2] = Tcl_NewStringObj("inherit", 7);
    else {
       for (i = 0; i < number_colors; i++)
          if (colorlist[i].color.pixel == colorval)
 	    break;
-      objv[1] = Tcl_NewIntObj((int)i);
+      objv[2] = Tcl_NewIntObj((int)i);
    }
-   Tcl_SetVar2Ex(xcinterp, "colorval", NULL, objv[1], TCL_NAMESPACE_ONLY);
+   Tcl_SetVar2Ex(xcinterp, "XCOps", "colorval", objv[2], TCL_NAMESPACE_ONLY);
 
-   XcTagCallback(xcinterp, 2, objv);
+   XcTagCallback(xcinterp, 3, objv);
 #endif
 }
 
@@ -1719,7 +1760,7 @@ void setallstylemarks(u_short styleval)
    /* Set GUI variables and execute any command tags associated	*/
    /* with the "fill" and "border" commands.			*/
 
-   Tcl_Obj *objv[2];
+   Tcl_Obj *objv[3];
    int fillfactor;
    const char *bptr;
 
@@ -1738,13 +1779,18 @@ void setallstylemarks(u_short styleval)
    else
       objv[1] = Tcl_NewStringObj("unfilled", 8);
 
-   Tcl_SetVar2Ex(xcinterp, "fillamount", NULL, objv[1], TCL_NAMESPACE_ONLY);
-   Tcl_SetVar(xcinterp, "opaque", (styleval & OPAQUE) ? "true" : "false",
+   if (styleval & OPAQUE)
+      objv[2] = Tcl_NewStringObj("opaque", 6);
+   else
+      objv[2] = Tcl_NewStringObj("transparent", 11);
+
+   Tcl_SetVar2Ex(xcinterp, "XCOps", "fillamount", objv[1], TCL_NAMESPACE_ONLY);
+   Tcl_SetVar2(xcinterp, "XCOps", "opaque", (styleval & OPAQUE) ? "true" : "false",
 		TCL_NAMESPACE_ONLY);
-   Tcl_SetVar(xcinterp, "bboxtype", (styleval & BBOX) ? "true" : "false",
-		TCL_NAMESPACE_ONLY);
-   Tcl_SetVar(xcinterp, "polyclosed", (styleval & UNCLOSED) ? "false" : "true",
+   Tcl_SetVar2(xcinterp, "XCOps", "bboxtype", (styleval & BBOX) ? "true" : "false",
 		TCL_NAMESPACE_ONLY);
+   Tcl_SetVar2(xcinterp, "XCOps", "polyclosed", (styleval & UNCLOSED) ? "false" :
+		"true", TCL_NAMESPACE_ONLY);
 
    switch (styleval & BORDERS) {
       case DASHED:
@@ -1760,9 +1806,9 @@ void setallstylemarks(u_short styleval)
 	 bptr = borders[SolidIdx];
 	 break;
    }
-   Tcl_SetVar(xcinterp, "linestyle", (char *)bptr, TCL_NAMESPACE_ONLY);
+   Tcl_SetVar2(xcinterp, "XCOps", "linestyle", (char *)bptr, TCL_NAMESPACE_ONLY);
 
-   XcTagCallback(xcinterp, 2, objv);
+   XcTagCallback(xcinterp, 3, objv);
 
 #endif
 }
@@ -1789,7 +1835,7 @@ polyptr checkforbbox(objectptr localdata)
 /* off.								*/
 /*--------------------------------------------------------------*/
 
-void setelementstyle(xcWidget w, u_short value, u_short mask)
+int setelementstyle(xcWidget w, u_short value, u_short mask)
 {
    Boolean selected = False;
    short *sstyle;
@@ -1800,16 +1846,16 @@ void setelementstyle(xcWidget w, u_short value, u_short mask)
 	 polyptr ckp;
 	 if (areastruct.selects != 1) {
 	    Wprintf("Choose only one polygon to be the bounding box");
-	    return;
+	    return -1;
 	 }
 	 else if (SELECTTYPE(areastruct.selectlist) != POLYGON) {
 	    Wprintf("Bounding box can only be a polygon");
-	    return;
+	    return -1;
 	 }
 	 else if (((ckp = checkforbbox(topobject)) != NULL) &&
 		(ckp != SELTOPOLY(areastruct.selectlist))) {
 	    Wprintf("Only one bounding box allowed per page");
-	    return;
+	    return -1;
 	 }
       }
 
@@ -1863,7 +1909,7 @@ void setelementstyle(xcWidget w, u_short value, u_short mask)
       if (value & BBOX) {
 	 Wprintf("Cannot set default style to Bounding Box");
 	 newstyle &= ~(BBOX);
-	 return;
+	 return -1;
       }
       else {
 	 newstyle &= ~mask;
@@ -1872,17 +1918,21 @@ void setelementstyle(xcWidget w, u_short value, u_short mask)
 
       if ((newstyle & NOBORDER) && !(newstyle & FILLED)) {
 	 Wprintf("Must have either a border or filler");
-	 return;
+	 return -1;
       }
       areastruct.style = newstyle;
       overdrawpixmap(w);
    }
+#ifndef TCL_WRAPPER
    setallstylemarks(newstyle);
    objectdeselect();
+#endif
+   return (int)newstyle;
 }
 
+#ifndef TCL_WRAPPER
 /*--------------------------------------------------------------*/
-/* The following four routines are all wrappers for		*/
+/* The following five routines are all wrappers for		*/
 /* setelementstyle(),	  					*/
 /* used in menudefs to differentiate between sections, each of  */
 /* which has settings independent of the others.		*/
@@ -1909,23 +1959,26 @@ void setclosure(xcWidget w, pointertype value, caddr_t calldata)
 
 /*----------------------------------------------------------------*/
 
-void setline(xcWidget w, pointertype value, caddr_t calldata)
-{
-   setelementstyle(w, (u_short)value, BORDERS);
-}
-
-/*----------------------------------------------------------------*/
-
 void setopaque(xcWidget w, pointertype value, caddr_t calldata)
 {
    setelementstyle(w, (u_short)value, OPAQUE);
 }
    
+#endif /* TCL_WRAPPER */
+
+/*----------------------------------------------------------------*/
+
+int setline(xcWidget w, pointertype value, caddr_t calldata)
+{
+   return setelementstyle(w, (u_short)value, BORDERS);
+}
+
 /*-----------------------------------------------*/
 /* Set the color value for all selected elements */
 /*-----------------------------------------------*/
 
 #ifdef TCL_WRAPPER
+
 void setcolor(xcWidget w, int cindex)
 {
    short *scolor;
@@ -2002,7 +2055,9 @@ void setcolor(xcWidget w, pointertype value, caddr_t calldata)
       overdrawpixmap(w);
    }
 
+#ifndef TCL_WRAPPER
    objectdeselect();
+#endif
 }
 
 /*----------------------------------------------------------------*/
@@ -2062,13 +2117,14 @@ void addnewcolor(xcWidget w, caddr_t clientdata, caddr_t calldata)
 void togglestylemark(int styleval)
 {
    switch(styleval) {
-      case 0: Tcl_SetVar(xcinterp, "fontstyle", "normal", TCL_NAMESPACE_ONLY);
+      case 0: Tcl_SetVar2(xcinterp, "XCOps", "fontstyle", "normal", TCL_NAMESPACE_ONLY);
 	 break;
-      case 1: Tcl_SetVar(xcinterp, "fontstyle", "bold", TCL_NAMESPACE_ONLY);
+      case 1: Tcl_SetVar2(xcinterp, "XCOps", "fontstyle", "bold", TCL_NAMESPACE_ONLY);
 	 break;
-      case 2: Tcl_SetVar(xcinterp, "fontstyle", "italic", TCL_NAMESPACE_ONLY);
+      case 2: Tcl_SetVar2(xcinterp, "XCOps", "fontstyle", "italic", TCL_NAMESPACE_ONLY);
 	 break;
-      case 3: Tcl_SetVar(xcinterp, "fontstyle", "bolditalic", TCL_NAMESPACE_ONLY);
+      case 3: Tcl_SetVar2(xcinterp, "XCOps", "fontstyle", "bolditalic",
+		TCL_NAMESPACE_ONLY);
 	 break;
    }
 }
@@ -2078,21 +2134,29 @@ void togglestylemark(int styleval)
 void toggleencodingmark(int encodingval)
 {
    switch(encodingval) {
-      case 0: Tcl_SetVar(xcinterp, "fontencoding", "Standard", TCL_NAMESPACE_ONLY);
+      case 0: Tcl_SetVar2(xcinterp, "XCOps", "fontencoding", "Standard",
+		TCL_NAMESPACE_ONLY);
 	 break;
-      case 1: Tcl_SetVar(xcinterp, "fontencoding", "special", TCL_NAMESPACE_ONLY);
+      case 1: Tcl_SetVar2(xcinterp, "XCOps", "fontencoding", "special",
+		TCL_NAMESPACE_ONLY);
 	 break;
-      case 2: Tcl_SetVar(xcinterp, "fontencoding", "ISOLatin1", TCL_NAMESPACE_ONLY);
+      case 2: Tcl_SetVar2(xcinterp, "XCOps", "fontencoding", "ISOLatin1",
+		TCL_NAMESPACE_ONLY);
 	 break;
-      case 3: Tcl_SetVar(xcinterp, "fontencoding", "ISOLatin2", TCL_NAMESPACE_ONLY);
+      case 3: Tcl_SetVar2(xcinterp, "XCOps", "fontencoding", "ISOLatin2",
+		TCL_NAMESPACE_ONLY);
 	 break;
-      case 4: Tcl_SetVar(xcinterp, "fontencoding", "ISOLatin3", TCL_NAMESPACE_ONLY);
+      case 4: Tcl_SetVar2(xcinterp, "XCOps", "fontencoding", "ISOLatin3",
+		TCL_NAMESPACE_ONLY);
 	 break;
-      case 5: Tcl_SetVar(xcinterp, "fontencoding", "ISOLatin4", TCL_NAMESPACE_ONLY);
+      case 5: Tcl_SetVar2(xcinterp, "XCOps", "fontencoding", "ISOLatin4",
+		TCL_NAMESPACE_ONLY);
 	 break;
-      case 6: Tcl_SetVar(xcinterp, "fontencoding", "ISOLatin5", TCL_NAMESPACE_ONLY);
+      case 6: Tcl_SetVar2(xcinterp, "XCOps", "fontencoding", "ISOLatin5",
+		TCL_NAMESPACE_ONLY);
 	 break;
-      case 7: Tcl_SetVar(xcinterp, "fontencoding", "ISOLatin6", TCL_NAMESPACE_ONLY);
+      case 7: Tcl_SetVar2(xcinterp, "XCOps", "fontencoding", "ISOLatin6",
+		TCL_NAMESPACE_ONLY);
 	 break;
    }
 }
@@ -2101,32 +2165,40 @@ void toggleencodingmark(int encodingval)
 
 void togglejustmarks(int justvalue)
 {
-   switch(justvalue & (NOTLEFT | RIGHT)) {
-      case NORMAL: Tcl_SetVar(xcinterp, "jhoriz", "left", TCL_NAMESPACE_ONLY);
+   switch(justvalue & RLJUSTFIELD) {
+      case NORMAL: Tcl_SetVar2(xcinterp, "XCOps", "jhoriz", "left",
+		TCL_NAMESPACE_ONLY);
 	 break;
-      case NOTLEFT: Tcl_SetVar(xcinterp, "jhoriz", "center", TCL_NAMESPACE_ONLY);
+      case NOTLEFT: Tcl_SetVar2(xcinterp, "XCOps", "jhoriz", "center",
+		TCL_NAMESPACE_ONLY);
 	 break;
-      case RIGHT|NOTLEFT: Tcl_SetVar(xcinterp, "jhoriz", "right", TCL_NAMESPACE_ONLY);
+      case RIGHT|NOTLEFT: Tcl_SetVar2(xcinterp, "XCOps", "jhoriz", "right",
+		TCL_NAMESPACE_ONLY);
 	 break;
    }
-   switch(justvalue & (NOTBOTTOM | TOP)) {
-      case NORMAL: Tcl_SetVar(xcinterp, "jvert", "bottom", TCL_NAMESPACE_ONLY);
+   switch(justvalue & TBJUSTFIELD) {
+      case NORMAL: Tcl_SetVar2(xcinterp, "XCOps", "jvert", "bottom",
+		TCL_NAMESPACE_ONLY);
 	 break;
-      case NOTBOTTOM: Tcl_SetVar(xcinterp, "jvert", "middle", TCL_NAMESPACE_ONLY);
+      case NOTBOTTOM: Tcl_SetVar2(xcinterp, "XCOps", "jvert", "middle",
+		TCL_NAMESPACE_ONLY);
 	 break;
-      case TOP|NOTBOTTOM: Tcl_SetVar(xcinterp, "jvert", "top", TCL_NAMESPACE_ONLY);
+      case TOP|NOTBOTTOM: Tcl_SetVar2(xcinterp, "XCOps", "jvert", "top",
+		TCL_NAMESPACE_ONLY);
 	 break;
    }
 
    /* Flip Invariance property */
-   Tcl_SetVar(xcinterp, "flipinvariant", (justvalue & FLIPINV) ? "true" : "false",
-	TCL_NAMESPACE_ONLY);
+   Tcl_SetVar2(xcinterp, "XCOps", "flipinvariant", (justvalue & FLIPINV) ?
+		"true" : "false", TCL_NAMESPACE_ONLY);
+
+   /* LaTeX mode property */
+   Tcl_SetVar2(xcinterp, "XCOps", "latexmode", (justvalue & LATEXLABEL) ?
+		"true" : "false", TCL_NAMESPACE_ONLY);
      
-#ifdef SCHEMA
    /* Pin visibility property */
-   Tcl_SetVar(xcinterp, "pinvisible", (justvalue & PINVISIBLE) ? "true" : "false",
-	TCL_NAMESPACE_ONLY);
-#endif
+   Tcl_SetVar2(xcinterp, "XCOps", "pinvisible", (justvalue & PINVISIBLE) ?
+		"true" : "false", TCL_NAMESPACE_ONLY);
 }
 
 /*------------------------------------------------------*/
@@ -2171,14 +2243,14 @@ void setfontmarks(short fvalue, short jvalue)
       togglefontmark(fvalue);
    }
    if (jvalue >= 0) {
-      switch(jvalue & (NOTLEFT | RIGHT)) {
+      switch(jvalue & (RLJUSTFIELD)) {
          case NORMAL: w = JustificationLeftJustifiedButton; break;
          case NOTLEFT: w = JustificationCenterJustifiedButton; break;
          case RIGHT|NOTLEFT: w = JustificationRightJustifiedButton; break;
       }
       toggleexcl(w, Justifs, XtNumber(Justifs));
 
-      switch(jvalue & (NOTBOTTOM | TOP)) {
+      switch(jvalue & (TBJUSTFIELD)) {
          case NORMAL: w = JustificationBottomJustifiedButton; break;
          case NOTBOTTOM: w = JustificationMiddleJustifiedButton; break;
          case TOP|NOTBOTTOM: w = JustificationTopJustifiedButton; break;
@@ -2194,7 +2266,6 @@ void setfontmarks(short fvalue, short jvalue)
          XtSetArg(wargs[0], XtNsetMark, False);
       XtSetValues(w, wargs, 1);
 
-#ifdef SCHEMA
       /* Pin visibility property */
       w = NetlistPinVisibilityButton;
       if (jvalue & PINVISIBLE)
@@ -2202,43 +2273,74 @@ void setfontmarks(short fvalue, short jvalue)
       else
          XtSetArg(wargs[0], XtNsetMark, False);
       XtSetValues(w, wargs, 1);
-#endif
    }
 }
 
+/*----------------------------------------------*/
+/* GUI wrapper for startparam()			*/
+/*----------------------------------------------*/
+
+void promptparam(xcWidget w, caddr_t clientdata, caddr_t calldata)
+{
+   buttonsave *popdata = (buttonsave *)malloc(sizeof(buttonsave));
+
+   if (areastruct.selects == 0) return;  /* nothing was selected */
+
+   /* Get a name for the new object */
+
+   eventmode = NORMAL_MODE;
+   popdata->dataptr = NULL;
+   popdata->button = NULL; /* indicates that no button is assc'd w/ the popup */
+   popupprompt(w, "Enter name for new parameter:", "\0", stringparam, popdata, NULL);
+}
+
 #endif /* !TCL_WRAPPER */
 
 /*--------------------------------------------------------------*/
 /* Parameterize a label string (wrapper for parameterize()).	*/
+/* Assumes that the name has been generated by the popup prompt	*/
+/* and is therefore held in variable _STR2			*/
+/*--------------------------------------------------------------*/
+
+void stringparam(xcWidget w, caddr_t clientdata, caddr_t calldata)
+{
+   genericptr *settext;
+
+      if (eventmode == TEXT2_MODE || eventmode == TEXT3_MODE) {
+         settext = (genericptr *)EDITPART;
+         makeparam(TOLABEL(settext), _STR2);
+         objectdeselect();
+	 setparammarks(NULL);
+      }
+      else if (checkselect(LABEL)) parameterize(P_SUBSTRING, _STR2);
+      else
+         eventmode = LPARAM_MODE;
+}
+
+/*--------------------------------------------------------------*/
+/* Numerical parameterization (wrapper for parameterize()).	*/
 /*--------------------------------------------------------------*/
 
 void startparam(xcWidget w, pointertype value, caddr_t calldata)
 {
    genericptr *settext;
 
-   if (value == P_SUBSTRING) {
-      if (eventmode == TEXT2_MODE || eventmode == TEXT3_MODE) {
-         settext = (genericptr *)EDITPART;
-         makeparam(TOLABEL(settext));
-         objectdeselect();
-	 setparammarks(NULL);
-      }
-      else if (checkselect(LABEL)) parameterize((int)value);
-      else
-         eventmode = LPARAM_MODE;
+   if (value == (pointertype)P_SUBSTRING) {
+      strcpy(_STR2, (calldata != NULL) ? (char *)calldata : "substring");
+      stringparam(w, NULL, NULL);
    }
    else {
       if ((eventmode == NORMAL_MODE) && (areastruct.selects == 0)) { 
          eventmode = IPARAM_MODE;
          Wprintf("Click on object to parameterize.");
       }      
-      else parameterize((int)value);
+      else parameterize((int)value, (char *)calldata);
    }
 }
 
-/*--------------------------------------------------------------*/
-/* Unparameterize a label string (wrapper for parameterize()).	*/
-/*--------------------------------------------------------------*/
+/*---------------------------------------------------------------*/
+/* Unparameterize a label string (wrapper for unparameterize()). */
+/*---------------------------------------------------------------*/
 
 void startunparam(xcWidget w, pointertype value, caddr_t calldata)
 {
@@ -2264,55 +2366,55 @@ void setdefaultfontmarks()
    setfontmarks(areastruct.psfont, areastruct.justify);
 }
 
-/*----------------------------------------------------------------*/
-/* Change flip invariance on the label passed as 2nd parameter	  */
-/*----------------------------------------------------------------*/
+/*--------------------------------------------------------------*/
+/* Toggle a bit in the justification field of a label		*/
+/*--------------------------------------------------------------*/
 
-void doflipflip(xcWidget w, labelptr settext)
+void dojustifybit(xcWidget w, labelptr settext, short bitfield)
 {
-   short newjust;
-   Boolean boolval;
-   u_short value;
-
    if (settext != NULL) {
-      boolval = (settext->justify & FLIPINV) ? True : False;
-      value = boolval ? 0 : FLIPINV;
-      newjust = (settext->justify & (~FLIPINV)) | value;
-
       undrawtext(settext);
-      settext->justify = newjust;
+      settext->justify ^= bitfield;
       redrawtext(settext);
       pwriteback(areastruct.topinstance);
    }
-   else {
-      boolval = (areastruct.justify & FLIPINV) ? True : False;
-      value = boolval ? 0 : FLIPINV;
-      newjust = (areastruct.justify & (~FLIPINV)) | value;
-      areastruct.justify = newjust;
+   else
+      areastruct.justify ^= bitfield;
+
+#ifndef TCL_WRAPPER
+   if (w != NULL) {
+      Boolean boolval;
+      if (settext)
+	 boolval = (settext->justify & bitfield) ? 0 : 1;
+      else
+	 boolval = (areastruct.justify & bitfield) ? 0 : 1;
+      toggle(w, &boolval, NULL);
    }
-   if (w != NULL) toggle(w, &boolval, NULL);
+#endif
 }
 
-/*----------------------------------------------------------------*/
-/* Change pin visibility on the label passed as 2nd parameter	  */
-/*----------------------------------------------------------------*/
+/*--------------------------------------------------------------*/
+/* Toggle a pin-related bit in the justification field of a	*/
+/* label.  This differs from dojustifybit() in that the label	*/
+/* must be a pin, and this function cannot change the default	*/
+/* behavior set by areastruct.justify.				*/
+/*--------------------------------------------------------------*/
 
-void dopinvisible(xcWidget w, labelptr settext)
+void dopinjustbit(xcWidget w, labelptr settext, short bitfield)
 {
-   short newjust;
-   Boolean boolval;
-   u_short value;
-
-   if (settext != NULL && settext->pin) {
-      boolval = (settext->justify & PINVISIBLE) ? True : False;
-      value = boolval ? 0 : PINVISIBLE;
-      newjust = (settext->justify & (~PINVISIBLE)) | value;
 
+   if ((settext != NULL) && settext->pin) {
       undrawtext(settext);
-      settext->justify = newjust;
+      settext->justify ^= bitfield;
       redrawtext(settext);
       pwriteback(areastruct.topinstance);
-      if (w != NULL) toggle(w, &boolval, NULL);
+
+#ifndef TCL_WRAPPER
+      if (w != NULL) {
+	 Boolean boolval = (settext->justify & bitfield) ? 0 : 1;
+	 toggle(w, &boolval, NULL);
+      }
+#endif
    }
 }
 
@@ -2327,10 +2429,10 @@ void setjust(xcWidget w, pointertype value, labelptr settext, short mode)
    if (settext != NULL) {
 
       if (mode == 1)
-         newjust = (settext->justify & (PINVISIBLE | FLIPINV | TOP | NOTBOTTOM))
+         newjust = (settext->justify & (NONJUSTFIELD | TBJUSTFIELD))
 			| value;
       else
-         newjust = (settext->justify & (PINVISIBLE | FLIPINV | RIGHT | NOTLEFT))
+         newjust = (settext->justify & (NONJUSTFIELD | RLJUSTFIELD))
 			| value;
 
       undrawtext(settext);
@@ -2340,10 +2442,10 @@ void setjust(xcWidget w, pointertype value, labelptr settext, short mode)
    }
    else {
       if (mode == 1)
-         newjust = (areastruct.justify & (PINVISIBLE | FLIPINV | TOP | NOTBOTTOM))
+         newjust = (areastruct.justify & (NONJUSTFIELD | TBJUSTFIELD))
 			| value;
       else
-         newjust = (areastruct.justify & (PINVISIBLE | FLIPINV | RIGHT | NOTLEFT))
+         newjust = (areastruct.justify & (NONJUSTFIELD | RLJUSTFIELD))
 			| value;
       areastruct.justify = newjust;
    }
@@ -2410,11 +2512,12 @@ void sethjust(xcWidget w, pointertype value, caddr_t nulldata)
    }
 }
 
-/*----------------------------------------------------------------*/
-/* Set flip invariance on all selected labels			  */ 
-/*----------------------------------------------------------------*/
+/*--------------------------------------------------------------*/
+/* Set a justify field bit on all selected labels		*/ 
+/* (flip invariance, latex mode, etc.)				*/
+/*--------------------------------------------------------------*/
 
-void setflipinv(xcWidget w, pointertype value, caddr_t nulldata)
+void setjustbit(xcWidget w, pointertype value, caddr_t nulldata)
 {
    short *fselect;
    labelptr settext;
@@ -2422,7 +2525,7 @@ void setflipinv(xcWidget w, pointertype value, caddr_t nulldata)
 
    if (eventmode == TEXT2_MODE || eventmode == TEXT3_MODE) {
       settext = *((labelptr *)EDITPART);
-      doflipflip(w, settext);
+      dojustifybit(w, settext, (short)value);
    }
    else {
       for (fselect = areastruct.selectlist; fselect < areastruct.selectlist +
@@ -2430,37 +2533,36 @@ void setflipinv(xcWidget w, pointertype value, caddr_t nulldata)
          if (SELECTTYPE(fselect) == LABEL) {
             labelcount++;
             settext = SELTOLABEL(fselect);
-            doflipflip(NULL, settext);
+            dojustifybit(NULL, settext, (short)value);
          }
       }
-      if (labelcount == 0) doflipflip(w, NULL);
+      if (labelcount == 0) dojustifybit(w, NULL, (short)value);
       else objectdeselect();
    }
 }
 
-/*----------------------------------------------------------------*/
-/* Set pin visibility on all selected pins			  */ 
-/*----------------------------------------------------------------*/
+/*--------------------------------------------------------------*/
+/* Set pin-related bit field of "justify" on all selected pins	*/ 
+/* (e.g., pin visibility)					*/
+/*--------------------------------------------------------------*/
 
-void setpinvisible(xcWidget w, pointertype value, caddr_t nulldata)
+void setpinjustbit(xcWidget w, pointertype value, caddr_t nulldata)
 {
    short *fselect;
    labelptr settext;
-   short labelcount = 0;
 
    if (eventmode == TEXT2_MODE || eventmode == TEXT3_MODE) {
       settext = *((labelptr *)EDITPART);
       if (settext->pin)
-	 dopinvisible(w, settext);
+	 dopinjustbit(w, settext, (short)value);
    }
    else {
       for (fselect = areastruct.selectlist; fselect < areastruct.selectlist +
             areastruct.selects; fselect++) {
          if (SELECTTYPE(fselect) == LABEL) {
-            labelcount++;
             settext = SELTOLABEL(fselect);
 	    if (settext->pin)
-               dopinvisible(NULL, settext);
+               dopinjustbit(NULL, settext, (short)value);
          }
       }
       objectdeselect();
@@ -2477,19 +2579,19 @@ void boxedit(xcWidget w, pointertype value, caddr_t nulldata)
 {
    switch (value) {
       case MANHATTAN:
-	 Tcl_SetVar(xcinterp, "polyedittype", "manhattan", TCL_NAMESPACE_ONLY);
+	 Tcl_SetVar2(xcinterp, "XCOps", "polyedittype", "manhattan", TCL_NAMESPACE_ONLY);
 	 break;
       case RHOMBOIDX:
-	 Tcl_SetVar(xcinterp, "polyedittype", "rhomboidx", TCL_NAMESPACE_ONLY);
+	 Tcl_SetVar2(xcinterp, "XCOps", "polyedittype", "rhomboidx", TCL_NAMESPACE_ONLY);
 	 break;
       case RHOMBOIDY:
-	 Tcl_SetVar(xcinterp, "polyedittype", "rhomboidy", TCL_NAMESPACE_ONLY);
+	 Tcl_SetVar2(xcinterp, "XCOps", "polyedittype", "rhomboidy", TCL_NAMESPACE_ONLY);
 	 break;
       case RHOMBOIDA:
-	 Tcl_SetVar(xcinterp, "polyedittype", "rhomboida", TCL_NAMESPACE_ONLY);
+	 Tcl_SetVar2(xcinterp, "XCOps", "polyedittype", "rhomboida", TCL_NAMESPACE_ONLY);
 	 break;
       case NORMAL:
-	 Tcl_SetVar(xcinterp, "polyedittype", "normal", TCL_NAMESPACE_ONLY);
+	 Tcl_SetVar2(xcinterp, "XCOps", "polyedittype", "normal", TCL_NAMESPACE_ONLY);
 	 break;
    }
 
@@ -2926,17 +3028,25 @@ void fontencoding(xcWidget w, pointertype value, caddr_t nulldata)
 
 /*----------------------------------------------*/
 /* Generate the table of special characters	*/
+/*						*/
+/* Return FALSE if the edited label is LaTeX-	*/
+/* compatible, since LaTeX text does not	*/
+/* usually contain special characters, but it	*/
+/* does contain a lot of backslashes.		*/
 /*----------------------------------------------*/
 
-void dospecial()
+Boolean dospecial()
 {
    labelptr curlabel;
    int cfont;
 
    curlabel = TOLABEL(EDITPART);
+   if (curlabel->justify & LATEXLABEL) return False;
+
    cfont = findcurfont(textpos, curlabel->string, areastruct.topinstance);
    composefontlib(cfont);
    startcatalog(NULL, FONTLIB, NULL);
+   return True;
 }
 
 /*-------------------------------------------------*/
