@@ -2267,6 +2267,9 @@ void pre_initialize()
    signal(SIGINT, dointr);
    printtime_id = 0;
 
+   xobjs.undostack = NULL;
+   xobjs.redostack = NULL;
+
    /* Set the temporary directory name as compiled, unless overridden by */
    /* environment variable "TMPDIR".					 */
 
@@ -2307,7 +2310,6 @@ void pre_initialize()
    /* xobjs.numlibs = LIBS - LIBRARY; */
    xobjs.numlibs = LIBS - LIBRARY - 1;
    xobjs.fontlib.number = 0;
-   xobjs.delbuffer.number = 0;
    xobjs.userlibs = (Library *) malloc(xobjs.numlibs * sizeof(Library));
    for (i = 0; i < xobjs.numlibs; i++) {
       xobjs.userlibs[i].library = (objectptr *) malloc(sizeof(objectptr));
@@ -2315,7 +2317,6 @@ void pre_initialize()
       xobjs.userlibs[i].number = 0;
    }
   
-   xobjs.delbuffer.library = (objectptr *) malloc(DELBUFSIZE * sizeof(objectptr));
    xobjs.pages = PAGES;
 
    fontcount = 0;
@@ -2997,7 +2998,7 @@ int main(int argc, char **argv)
    /*---------------------------*/
 
    for (k = argc - 1; k > 0; k--) {
-      if (!strncmp(argv[k], "-schem", 6)) {
+      if (!strncmp(argv[k], SCHEM_FLAG, strlen(SCHEM_FLAG))) {
 	 areastruct.schemon = True;
 	 break;
       }
